@@ -13,12 +13,10 @@
 
 """Implementation of a number of stream filters."""
 
-try:
-    any
-except NameError:
-    from genshi.util import any
 import re
 
+import six
+
 from genshi.core import Attrs, QName, stripentities
 from genshi.core import END, START, TEXT, COMMENT
 
@@ -34,7 +32,7 @@ class HTMLFormFiller(object):
     ...   <p><input type="text" name="foo" /></p>
     ... </form>''', encoding='utf-8')
     >>> filler = HTMLFormFiller(data={'foo': 'bar'})
-    >>> print((html | filler))
+    >>> print(html | filler)
     <form>
       <p><input type="text" name="foo" value="bar"/></p>
     </form>
@@ -101,13 +99,13 @@ class HTMLFormFiller(object):
                                 checked = False
                                 if isinstance(value, (list, tuple)):
                                     if declval is not None:
-                                        checked = declval in [str(v) for v
-                                                              in value]
+                                        u_vals = [six.text_type(v) for v in value]
+                                        checked = declval in u_vals
                                     else:
                                         checked = any(value)
                                 else:
                                     if declval is not None:
-                                        checked = declval == str(value)
+                                        checked = declval == six.text_type(value)
                                     elif type == 'checkbox':
                                         checked = bool(value)
                                 if checked:
@@ -123,7 +121,7 @@ class HTMLFormFiller(object):
                                     value = value[0]
                                 if value is not None:
                                     attrs |= [
-                                        (QName('value'), str(value))
+                                        (QName('value'), six.text_type(value))
                                     ]
                     elif tagname == 'select':
                         name = attrs.get('name')
@@ -166,10 +164,10 @@ class HTMLFormFiller(object):
                     select_value = None
                 elif in_select and tagname == 'option':
                     if isinstance(select_value, (tuple, list)):
-                        selected = option_value in [str(v) for v
+                        selected = option_value in [six.text_type(v) for v
                                                     in select_value]
                     else:
-                        selected = option_value == str(select_value)
+                        selected = option_value == six.text_type(select_value)
                     okind, (tag, attrs), opos = option_start
                     if selected:
                         attrs |= [(QName('selected'), 'selected')]
@@ -185,7 +183,7 @@ class HTMLFormFiller(object):
                     option_text = []
                 elif in_textarea and tagname == 'textarea':
                     if textarea_value:
-                        yield TEXT, str(textarea_value), pos
+                        yield TEXT, six.text_type(textarea_value), pos
                         textarea_value = None
                     in_textarea = False
                 yield kind, data, pos
@@ -200,7 +198,7 @@ class HTMLSanitizer(object):
     
     >>> from genshi import HTML
     >>> html = HTML('<div><script>alert(document.cookie)</script></div>', encoding='utf-8')
-    >>> print((html | HTMLSanitizer()))
+    >>> print(html | HTMLSanitizer())
     <div/>
     
     The default set of safe tags and attributes can be modified when the filter
@@ -209,14 +207,14 @@ class HTMLSanitizer(object):
     
     >>> html = HTML('<div style="background: #000"></div>', encoding='utf-8')
     >>> sanitizer = HTMLSanitizer(safe_attrs=HTMLSanitizer.SAFE_ATTRS | set(['style']))
-    >>> print((html | sanitizer))
+    >>> print(html | sanitizer)
     <div style="background: #000"/>
     
     Note that even in this case, the filter *does* attempt to remove dangerous
     constructs from style attributes:
 
     >>> html = HTML('<div style="background: url(javascript:void); color: #000"></div>', encoding='utf-8')
-    >>> print((html | sanitizer))
+    >>> print(html | sanitizer)
     <div style="color: #000"/>
     
     This handles HTML entities, unicode escapes in CSS and Javascript text, as
@@ -239,7 +237,7 @@ class HTMLSanitizer(object):
         'hr', 'i', 'img', 'input', 'ins', 'kbd', 'label', 'legend', 'li', 'map',
         'menu', 'ol', 'optgroup', 'option', 'p', 'pre', 'q', 's', 'samp',
         'select', 'small', 'span', 'strike', 'strong', 'sub', 'sup', 'table',
-        'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'tr', 'tt', '',
+        'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'tr', 'tt', 'u',
         'ul', 'var'])
 
     SAFE_ATTRS = frozenset(['abbr', 'accept', 'accept-charset', 'accesskey',
@@ -311,7 +309,7 @@ class HTMLSanitizer(object):
         # The set of URI schemes that are considered safe.
 
     # IE6 <http://heideri.ch/jso/#80>
-    _EXPRESSION_SEARCH = re.compile("""
+    _EXPRESSION_SEARCH = re.compile(u"""
         [eE
          \uFF25 # FULLWIDTH LATIN CAPITAL LETTER E
          \uFF45 # FULLWIDTH LATIN SMALL LETTER E
@@ -356,7 +354,7 @@ class HTMLSanitizer(object):
     # IE6 <http://openmya.hacker.jp/hasegawa/security/expression.txt>
     #     7) Particular bit of Unicode characters
     _URL_FINDITER = re.compile(
-        '[Uu][Rr\u0280][Ll\u029F]\s*\(([^)]+)').finditer
+        u'[Uu][Rr\u0280][Ll\u029F]\s*\(([^)]+)').finditer
 
     def __call__(self, stream):
         """Apply the filter to the given stream.
@@ -474,7 +472,7 @@ class HTMLSanitizer(object):
         that is not considered safe are removed:
         
         >>> sanitizer = HTMLSanitizer()
-        >>> sanitizer.sanitize_css('''
+        >>> sanitizer.sanitize_css(u'''
         ...   background: url(javascript:alert("foo"));
         ...   color: #000;
         ... ''')
@@ -483,7 +481,7 @@ class HTMLSanitizer(object):
         Also, the proprietary Internet Explorer function ``expression()`` is
         always stripped:
         
-        >>> sanitizer.sanitize_css('''
+        >>> sanitizer.sanitize_css(u'''
         ...   background: #fff;
         ...   color: #000;
         ...   width: e/**/xpression(alert("foo"));
@@ -528,7 +526,7 @@ class HTMLSanitizer(object):
         def _repl(match):
             t = match.group(1)
             if t:
-                return chr(int(t, 16))
+                return six.unichr(int(t, 16))
             t = match.group(2)
             if t == '\\':
                 return r'\\'
@@ -540,4 +538,3 @@ class HTMLSanitizer(object):
 
     def _strip_css_comments(self, text):
         return self._CSS_COMMENTS('', text)
-
